@@ -40,8 +40,6 @@ gboolean bitlbee_io_new_client( GIOChannel *source, GIOCondition condition, gpoi
 	int new_socket = accept( global.listen_socket, (struct sockaddr *) &conn_info, 
 		                     &size );
 	
-	count_io_event(source, "main");
-	
 	log_message( LOGLVL_INFO, "Creating new connection with fd %d.", new_socket );
 	irc_new( new_socket );
 
@@ -124,15 +122,19 @@ gboolean bitlbee_io_current_client_read( GIOChannel *source, GIOCondition condit
 	char line[513];
 	int st;
 	
-	count_io_event(source, "main");
-
-	if (condition & G_IO_ERR || condition & G_IO_HUP ) {
+	if( condition & G_IO_ERR || condition & G_IO_HUP )
+	{
 		irc_free( irc );
 		return FALSE;
 	}
-
+	
 	st = read( irc->fd, line, sizeof( line ) - 1 );
-	if( st <= 0 )
+	if( st == 0 )
+	{
+		irc_free( irc );
+		return FALSE;
+	}
+	else if( st < 0 )
 	{
 		if( sockerr_again() )
 		{
@@ -144,6 +146,7 @@ gboolean bitlbee_io_current_client_read( GIOChannel *source, GIOCondition condit
 			return FALSE;
 		}
 	}
+	
 	line[st] = '\0';
 	if( irc->readbuffer == NULL ) 
 	{
@@ -161,7 +164,15 @@ gboolean bitlbee_io_current_client_read( GIOChannel *source, GIOCondition condit
 		irc_free( irc );
 		return FALSE;
 	} 
-		
+	
+	/* Very naughty, go read the RFCs! >:) */
+	if( irc->readbuffer && ( strlen( irc->readbuffer ) > 1024 ) )
+	{
+		log_message( LOGLVL_ERROR, "Maximum line length exceeded." );
+		irc_free( irc );
+		return FALSE;
+	}
+	
 	return TRUE;
 }
 
@@ -170,34 +181,12 @@ gboolean bitlbee_io_current_client_write( GIOChannel *source, GIOCondition condi
 	irc_t *irc = data;
 	int st, size;
 	char *temp;
-#ifdef FLOOD_SEND
-	time_t newtime;
-#endif
 
-	count_io_event(source, "main");
-
-#ifdef FLOOD_SEND	
-	newtime = time( NULL );
-	if( ( newtime - irc->oldtime ) > FLOOD_SEND_INTERVAL )
-	{
-		irc->sentbytes = 0;
-		irc->oldtime = newtime;
-	}
-#endif
-	
 	if( irc->sendbuffer == NULL )
 		return( FALSE );
 	
 	size = strlen( irc->sendbuffer );
-	
-#ifdef FLOOD_SEND
-	if( ( FLOOD_SEND_BYTES - irc->sentbytes ) > size )
-		st = write( irc->fd, irc->sendbuffer, size );
-	else
-		st = write( irc->fd, irc->sendbuffer, ( FLOOD_SEND_BYTES - irc->sentbytes ) );
-#else
 	st = write( irc->fd, irc->sendbuffer, size );
-#endif
 	
 	if( st <= 0 )
 	{
@@ -212,10 +201,6 @@ gboolean bitlbee_io_current_client_write( GIOChannel *source, GIOCondition condi
 		}
 	}
 	
-#ifdef FLOOD_SEND
-	irc->sentbytes += st;
-#endif		
-	
 	if( st == size )
 	{
 		g_free( irc->sendbuffer );
@@ -234,7 +219,7 @@ gboolean bitlbee_io_current_client_write( GIOChannel *source, GIOCondition condi
 	}
 }
 
-int bitlbee_load( irc_t *irc, char* password )
+int bitlbee_load( irc_t *irc, const char* password )
 {
 	char s[512];
 	char *line;
@@ -252,7 +237,10 @@ int bitlbee_load( irc_t *irc, char* password )
 	
 	fscanf( fp, "%32[^\n]s", s );
 	if( setpass( irc, password, s ) < 0 )
+	{
+		fclose( fp );
 		return( -1 );
+	}
 	
 	/* Do this now. If the user runs with AuthMode = Registered, the
 	   account command will not work otherwise. */
@@ -262,7 +250,7 @@ int bitlbee_load( irc_t *irc, char* password )
 	{
 		fgetc( fp );
 		line = deobfucrypt( irc, s );
-		root_command_string( irc, ru, line );
+		root_command_string( irc, ru, line, 0 );
 		g_free( line );
 	}
 	fclose( fp );
@@ -280,7 +268,7 @@ int bitlbee_load( irc_t *irc, char* password )
 	if( set_getint( irc, "auto_connect" ) )
 	{
 		strcpy( s, "account on" );	/* Can't do this directly because r_c_s alters the string */
-		root_command_string( irc, ru, s );
+		root_command_string( irc, ru, s, 0 );
 	}
 	
 	return( 1 );
@@ -336,7 +324,11 @@ int bitlbee_save( irc_t *irc )
 			return( 0 );
 		}
 	}
-	fclose( fp );
+	if( fclose( fp ) != 0 )
+	{
+		irc_usermsg( irc, "fclose() reported an error. Disk full?" );
+		return( 0 );
+	}
   
 	g_snprintf( new_path, 512, "%s%s%s", global.conf->configdir, irc->nick, ".nicks" );
 	if( unlink( new_path ) != 0 )
@@ -419,7 +411,11 @@ int bitlbee_save( irc_t *irc )
 		}
 		g_free( line );
 	}
-	fclose( fp );
+	if( fclose( fp ) != 0 )
+	{
+		irc_usermsg( irc, "fclose() reported an error. Disk full?" );
+		return( 0 );
+	}
 	
  	g_snprintf( new_path, 512, "%s%s%s", global.conf->configdir, irc->nick, ".accounts" );
  	if( unlink( new_path ) != 0 )
@@ -451,7 +447,7 @@ void bitlbee_shutdown( gpointer data )
 	g_main_quit( global.loop );
 }
 
-int root_command_string( irc_t *irc, user_t *u, char *command )
+int root_command_string( irc_t *irc, user_t *u, char *command, int flags )
 {
 	char *cmd[IRC_MAX_ARGS];
 	char *s;
@@ -505,79 +501,7 @@ int root_command( irc_t *irc, char *cmd[] )
 			return( 1 );
 		}
 	
-	irc_usermsg( irc, "Unknown command: %s. Please use help commands to get a list of available commands.", cmd[0] );
+	irc_usermsg( irc, "Unknown command: %s. Please use \x02help commands\x02 to get a list of available commands.", cmd[0] );
 	
 	return( 1 );
 }
-
-/* Decode%20a%20file%20name						*/
-void http_decode( char *s )
-{
-	char *t;
-	int i, j, k;
-	
-	t = g_new( char, strlen( s ) + 1 );
-	
-	for( i = j = 0; s[i]; i ++, j ++ )
-	{
-		if( s[i] == '%' )
-		{
-			if( sscanf( s + i + 1, "%2x", &k ) )
-			{
-				t[j] = k;
-				i += 2;
-			}
-			else
-			{
-				*t = 0;
-				break;
-			}
-		}
-		else
-		{
-			t[j] = s[i];
-		}
-	}
-	t[j] = 0;
-	
-	strcpy( s, t );
-	g_free( t );
-}
-
-/* Warning: This one explodes the string. Worst-cases can make the string 3x its original size! */
-/* This fuction is safe, but make sure you call it safely as well! */
-void http_encode( char *s )
-{
-	char *t;
-	int i, j;
-	
-	t = g_strdup( s );
-	
-	for( i = j = 0; t[i]; i ++, j ++ )
-	{
-		if( t[i] <= ' ' || ((unsigned char *)t)[i] >= 128 || t[i] == '%' )
-		{
-			sprintf( s + j, "%%%02X", ((unsigned char*)t)[i] );
-			j += 2;
-		}
-		else
-		{
-			s[j] = t[i];
-		}
-	}
-	s[j] = 0;
-	
-	g_free( t );
-}
-
-/* Strip newlines from a string. Modifies the string passed to it. */ 
-char *strip_newlines( char *source )
-{
-	int i;	
-
-	for( i = 0; source[i] != '\0'; i ++ )
-		if( source[i] == '\n' || source[i] == '\r' )
-			source[i] = 32;
-	
-	return source;
-}
