@@ -29,12 +29,14 @@
 #define _GNU_SOURCE /* Stupid GNU :-P */
 
 #define PACKAGE "BitlBee"
-#define BITLBEE_VERSION "0.92"
+#define BITLBEE_VERSION "1.0.1"
 #define VERSION BITLBEE_VERSION
 
 #define MAX_STRING 128
 
+#if HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <fcntl.h>
 #include <time.h>
@@ -73,16 +75,20 @@
 #undef strncasecmp
 #define strncasecmp	__PLEASE_USE_G_STRNCASECMP_INSTEAD__
 
+#ifndef F_OK
+#define F_OK 0
+#endif
+
 #define _( x ) x
 
 #define ROOT_NICK "root"
-#define ROOT_CHAN "#bitlbee"
+#define ROOT_CHAN "&bitlbee"
 #define ROOT_FN "User manager"
 
 #define NS_NICK "NickServ"
 
 #define DEFAULT_AWAY "Away from computer"
-#define CONTROL_TOPIC "Welcome to the control channel. Type help for help information."
+#define CONTROL_TOPIC "Welcome to the control channel. Type \2help\2 for help information."
 #define IRCD_INFO "BitlBee <http://www.bitlbee.org/>"
 
 #define MAX_NICK_LENGTH 24
@@ -102,7 +108,6 @@ extern char *CONF_FILE;
 #include "ini.h"
 #include "help.h"
 #include "query.h"
-#include "debug.h"
 #include "sock.h"
 
 typedef struct global_t {
@@ -119,9 +124,9 @@ int bitlbee_inetd_init( void );
 gboolean bitlbee_io_current_client_read( GIOChannel *source, GIOCondition condition, gpointer data );
 gboolean bitlbee_io_current_client_write( GIOChannel *source, GIOCondition condition, gpointer data );
 
-int root_command_string( irc_t *irc, user_t *u, char *command );
+int root_command_string( irc_t *irc, user_t *u, char *command, int flags );
 int root_command( irc_t *irc, char *command[] );
-int bitlbee_load( irc_t *irc, char *password );
+int bitlbee_load( irc_t *irc, const char *password );
 int bitlbee_save( irc_t *irc );
 void bitlbee_shutdown( gpointer data );
 double gettime( void );
