@@ -51,7 +51,6 @@ command_t commands[] = {
 	{ "nick",           1, cmd_nick },
 	{ "import_buddies", 1, cmd_import_buddies },
 	{ "qlist",          0, cmd_qlist },
-	{ "dump",	    0, cmd_dump },
 	{ NULL }
 };
 
@@ -128,7 +127,7 @@ int cmd_register( irc_t *irc, char **cmd )
 	if( checkie == -2 )
 	{
 		setpassnc( irc, cmd[1] );
-		root_command_string( irc, user_find( irc, irc->mynick ), "save" );
+		root_command_string( irc, user_find( irc, irc->mynick ), "save", 0 );
 		irc->status = USTATUS_IDENTIFIED;
 	}
 	else
@@ -337,7 +336,7 @@ int cmd_account( irc_t *irc, char **cmd )
 	}
 	else
 	{
-		irc_usermsg( irc, "Unknown command: account %s. Please use help commands to get a list of available commands.", cmd[1] );
+		irc_usermsg( irc, "Unknown command: account %s. Please use \x02help commands\x02 to get a list of available commands.", cmd[1] );
 	}
 	
 	return( 1 );
@@ -738,7 +737,10 @@ int cmd_qlist( irc_t *irc, char **cmd )
 	irc_usermsg( irc, "Pending queries:" );
 	
 	for( num = 0; q; q = q->next, num ++ )
-		irc_usermsg( irc, "%d, %s", num, q->question );
+		if( q->gc ) /* Not necessary yet, but it might come later */
+			irc_usermsg( irc, "%d, %s(%s): %s", num, proto_name[q->gc->protocol], q->gc->username, q->question );
+		else
+			irc_usermsg( irc, "%d, BitlBee: %s", num, q->question );
 	
 	return( 0 );
 }
@@ -795,10 +797,3 @@ int cmd_import_buddies( irc_t *irc, char **cmd )
 	
 	return( 0 );
 }
-
-int cmd_dump( irc_t *irc, char **cmd ) {
-	write_io_activity();
-	irc_usermsg(irc, "Wrote GIO activity log to disk.");	
-	
-	return( 0 );
-}
