@@ -114,18 +114,18 @@ irc_t *irc_new( int fd )
 	set_add( irc, "auto_reconnect", "false", set_eval_bool );
 	set_add( irc, "auto_reconnect_delay", "300", set_eval_int );
 	set_add( irc, "buddy_sendbuffer", "false", set_eval_bool );
-	set_add( irc, "buddy_sendbuffer_delay", "1", set_eval_int );
+	set_add( irc, "buddy_sendbuffer_delay", "200", set_eval_int );
 	set_add( irc, "charset", "iso8859-1", set_eval_charset );
 	set_add( irc, "debug", "false", set_eval_bool );
 	set_add( irc, "default_target", "root", NULL );
 	set_add( irc, "display_namechanges", "false", set_eval_bool );
 	set_add( irc, "handle_unknown", "root", NULL );
-	set_add( irc, "html", "nostrip", NULL );
 	set_add( irc, "lcnicks", "true", set_eval_bool );
 	set_add( irc, "ops", "both", set_eval_ops );
 	set_add( irc, "private", "true", set_eval_bool );
 	set_add( irc, "query_order", "lifo", NULL );
-	set_add( irc, "save_on_quit", "1", set_eval_bool );
+	set_add( irc, "save_on_quit", "true", set_eval_bool );
+	set_add( irc, "strip_html", "true", NULL );
 	set_add( irc, "to_char", ": ", set_eval_to_char );
 	set_add( irc, "typing_notice", "false", set_eval_bool );
 	
@@ -161,7 +161,7 @@ void irc_free(irc_t * irc)
 	g_source_remove( irc->r_watch_source_id );
 	if( irc->w_watch_source_id > 0 )
 		g_source_remove( irc->w_watch_source_id );
-	g_io_channel_close( irc->io_channel );
+	
 	g_io_channel_unref( irc->io_channel );
 	irc_connection_list = g_slist_remove( irc_connection_list, irc );
 	
@@ -289,7 +289,8 @@ int irc_process( irc_t *irc )
 	return 1;	
 }
 
-char **irc_tokenize( char *buffer ) {
+char **irc_tokenize( char *buffer )
+{
 	int i, j;
 	char **lines;
 
@@ -461,8 +462,18 @@ int irc_exec( irc_t *irc, char **cmd )
 		}
 		return( 1 );
 	}
+	else if( g_strcasecmp( cmd[0], "QUIT" ) == 0 )
+	{
+		irc_write( irc, "ERROR :%s%s", cmd[1]?"Quit: ":"", cmd[1]?cmd[1]:"Client Quit" );
+		g_io_channel_close( irc->io_channel );
+		return( 0 );
+	}
 	
-	if( !irc->user || !irc->nick ) return( 1 );
+	if( !irc->user || !irc->nick )
+	{
+		irc_reply( irc, 451, ":Register first" );
+		return( 1 );
+	}
 	
 	if( g_strcasecmp( cmd[0], "PING" ) == 0 )
 	{
@@ -474,7 +485,7 @@ int irc_exec( irc_t *irc, char **cmd )
 		{
 			irc_reply( irc, 461, "%s :Need more parameters", cmd[0] );
 		}
-		else if( *cmd[1] == '#' )
+		else if( *cmd[1] == '#' || *cmd[1] == '&' )
 		{
 			if( cmd[2] )
 			{
@@ -545,7 +556,7 @@ int irc_exec( irc_t *irc, char **cmd )
 			     RFC doesn't have any reply for that though? */
 		else if( cmd[1] )
 		{
-			if( cmd[1][0] == '#' && cmd[1][1] )
+			if( ( cmd[1][0] == '#' || cmd[1][0] == '&' ) && cmd[1][1] )
 			{
 				user_t *u = user_find( irc, cmd[1] + 1 );
 				
@@ -576,13 +587,21 @@ int irc_exec( irc_t *irc, char **cmd )
 		else
 			irc_reply( irc, 461, "%s :Need more parameters", cmd[0] );
 	}
-	else if( g_strcasecmp( cmd[0], "PRIVMSG" ) == 0 )
+	else if( g_strcasecmp( cmd[0], "PRIVMSG" ) == 0 || g_strcasecmp( cmd[0], "NOTICE" ) == 0 )
 	{
-		if( !( cmd[1] && cmd[2] ) )
+		if( !cmd[1] )
 		{
 			irc_reply( irc, 461, "%s :Need more parameters", cmd[0] );
+		} 
+		else if ( !cmd[2] ) 
+		{
+			irc_reply( irc, 412, ":No text to send" );
 		}
-		else
+		else if ( irc->nick && g_strcasecmp( cmd[1], irc->nick ) == 0 ) 
+		{
+			irc_write( irc, ":%s!%s@%s %s %s :%s", irc->nick, irc->user, irc->host, cmd[0], cmd[1], cmd[2] ); 
+		}
+		else 
 		{
 			if( g_strcasecmp( cmd[1], irc->channel ) == 0 )
 			{
@@ -620,27 +639,26 @@ int irc_exec( irc_t *irc, char **cmd )
 			{
 				irc->is_private = 1;
 			}
-			irc_send( irc, cmd[1], cmd[2] );
+			irc_send( irc, cmd[1], cmd[2], ( g_strcasecmp( cmd[0], "NOTICE" ) == 0 ) ? IM_FLAG_AWAY : 0 );
 		}
 	}
-	else if( g_strcasecmp( cmd[0], "QUIT" ) == 0 )
-	{
-		irc_write( irc, "ERROR :%s%s", cmd[1]?"Quit: ":"", cmd[1]?cmd[1]:"Client Quit" );
-		g_io_channel_close( irc->io_channel );
-		return( 0 );
-	}
 	else if( g_strcasecmp( cmd[0], "WHO" ) == 0 )
 	{
 		irc_who( irc, cmd[1] );
 	}
 	else if( g_strcasecmp( cmd[0], "USERHOST" ) == 0 )
 	{
+		user_t *u;
+		
+		if( !cmd[1] )
+		{
+			irc_reply( irc, 461, "%s :Need more parameters", cmd[0] );
+		}
 		/* [TV] Usable USERHOST-implementation according to
 			RFC1459. Without this, mIRC shows an error
 			while connecting, and the used way of rejecting
 			breaks standards.
 		*/
-		user_t *u;
 		
 		for( i = 1; cmd[i]; i ++ )
 			if( ( u = user_find( irc, cmd[i] ) ) )
@@ -883,21 +901,6 @@ void irc_vawrite( irc_t *irc, char *format, va_list params )
 
 	if( irc->sendbuffer != NULL ) {
 		size = strlen( irc->sendbuffer ) + strlen( line );
-#ifdef FLOOD_SEND
-		if( size > FLOOD_SEND_MAXBUFFER ) {
-			/* Die flooder, die! >:) */
-
-			g_free(irc->sendbuffer);
-			
-			/* We need the \r\n at the start because else we might append our string to a half
-			 * sent line. A bit hackish, but it works.
-			 */
-			irc->sendbuffer = g_strdup( "\r\nERROR :Sendq Exceeded\r\n" );
-			irc->quit = 1;
-			
-			return;
-		}
-#endif
 		irc->sendbuffer = g_renew ( char, irc->sendbuffer, size + 1 );
 		strcpy( ( irc->sendbuffer + strlen( irc->sendbuffer ) ), line );
 	}
@@ -912,19 +915,31 @@ void irc_vawrite( irc_t *irc, char *format, va_list params )
 	return;
 }
 
-void irc_write_all( char *format, ... )
+void irc_write_all( int now, char *format, ... )
 {
 	va_list params;
 	GSList *temp;	
-
+	
 	va_start( params, format );
-
+	
 	temp = irc_connection_list;
-	while( temp!=NULL ) {
+	while( temp != NULL )
+	{
+		irc_t *irc = temp->data;
+		
+		if( now )
+		{
+			g_free( irc->sendbuffer );
+			irc->sendbuffer = g_strdup( "\r\n" );
+		}
 		irc_vawrite( temp->data, format, params );
+		if( now )
+		{
+			bitlbee_io_current_client_write( irc->io_channel, G_IO_OUT, irc );
+		}
 		temp = temp->next;
 	}
-
+	
 	va_end( params );
 	return;
 } 
@@ -1024,6 +1039,7 @@ void irc_login( irc_t *irc )
 	irc_reply( irc,   2, ":Host %s is running BitlBee " BITLBEE_VERSION " " ARCH "/" CPU ".", irc->myhost );
 	irc_reply( irc,   3, ":%s", IRCD_INFO );
 	irc_reply( irc,   4, "%s %s %s %s", irc->myhost, BITLBEE_VERSION, UMODES, CMODES );
+	irc_reply( irc,   5, "PREFIX=(ov)@+ CHANTYPES=#& CHANMODES=,,,%s NICKLEN=%d NETWORK=BitlBee CASEMAPPING=rfc1459 MAXTARGETS=1 WATCH=128 :are supported by this server", CMODES, MAX_NICK_LENGTH - 1 );
 	irc_motd( irc );
 	irc_umode_set( irc, irc->myhost, "+" UMODE );
 
@@ -1050,7 +1066,7 @@ void irc_login( irc_t *irc )
 //	u->send_handler = msg_echo;
 	irc_spawn( irc, u );
 	
-	irc_usermsg( irc, "Welcome to the BitlBee gateway!\n\nIf you've never used BitlBee before, please do read the help information using the help command. Lots of FAQ's are answered there." );
+	irc_usermsg( irc, "Welcome to the BitlBee gateway!\n\nIf you've never used BitlBee before, please do read the help information using the \x02help\x02 command. Lots of FAQ's are answered there." );
 	
 	irc->status = USTATUS_LOGGED_IN;
 }
@@ -1289,12 +1305,12 @@ void irc_invite( irc_t *irc, char *nick, char *channel )
 	irc_reply( irc, 482, "%s :Invite impossible; User/Channel non-existent or incompatible", channel );
 }
 
-int irc_send( irc_t *irc, char *nick, char *s )
+int irc_send( irc_t *irc, char *nick, char *s, int flags )
 {
 	struct conversation *c = NULL;
 	user_t *u = NULL;
 	
-	if( *nick == '#' )
+	if( *nick == '#' || *nick == '&' )
 	{
 		if( !( c = conv_findchannel( nick ) ) )
 		{
@@ -1377,7 +1393,7 @@ int irc_send( irc_t *irc, char *nick, char *s )
 		}
 		
 		if( u->send_handler )
-			return( u->send_handler( irc, u, s ) );
+			return( u->send_handler( irc, u, s, flags ) );
 	}
 	else if( c && c->gc && c->gc->prpl )
 	{
@@ -1392,27 +1408,38 @@ gboolean buddy_send_handler_delayed( gpointer data )
 	user_t *u = data;
 	
 	u->sendbuf[u->sendbuf_len-2] = 0; /* Cut off the last newline */
-	serv_send_im( u->gc->irc, u, u->sendbuf );
+	serv_send_im( u->gc->irc, u, u->sendbuf, u->sendbuf_flags );
 	
 	g_free( u->sendbuf );
 	u->sendbuf = NULL;
 	u->sendbuf_len = 0;
 	u->sendbuf_timer = 0;
+	u->sendbuf_flags = 0;
 	
 	return( FALSE );
 }
 
-int buddy_send_handler( irc_t *irc, user_t *u, char *msg )
+int buddy_send_handler( irc_t *irc, user_t *u, char *msg, int flags )
 {
 	if( !u || !u->gc ) return( 0 );
 	
 	if( set_getint( irc, "buddy_sendbuffer" ) && set_getint( irc, "buddy_sendbuffer_delay" ) > 0 )
 	{
+		int delay;
+		
+		if( u->sendbuf_len > 0 && u->sendbuf_flags != flags)
+		{
+			//Flush the buffer
+			g_source_remove( u->sendbuf_timer );
+			buddy_send_handler_delayed( u );
+		}
+
 		if( u->sendbuf_len == 0 )
 		{
 			u->sendbuf_len = strlen( msg ) + 2;
 			u->sendbuf = g_new (char, u->sendbuf_len );
 			u->sendbuf[0] = 0;
+			u->sendbuf_flags = flags;
 		}
 		else
 		{
@@ -1423,16 +1450,19 @@ int buddy_send_handler( irc_t *irc, user_t *u, char *msg )
 		strcat( u->sendbuf, msg );
 		strcat( u->sendbuf, "\n" );
 		
+		delay = set_getint( irc, "buddy_sendbuffer_delay" );
+		if( delay <= 5 )
+			delay *= 1000;
+		
 		if( u->sendbuf_timer > 0 )
 			g_source_remove( u->sendbuf_timer );
-		u->sendbuf_timer = g_timeout_add( set_getint( irc, "buddy_sendbuffer_delay" ) * 1000,
-		                                  buddy_send_handler_delayed, u );
+		u->sendbuf_timer = g_timeout_add( delay, buddy_send_handler_delayed, u );
 		
 		return( 1 );
 	}
 	else
 	{
-		return( serv_send_im( irc, u, msg ) );
+		return( serv_send_im( irc, u, msg, flags ) );
 	}
 }
 
@@ -1465,7 +1495,7 @@ int irc_privmsg( irc_t *irc, user_t *u, char *type, char *to, char *prefix, char
 			else
 			{
 				irc_write( irc, ":%s!%s@%s %s %s :%s%s", u->nick, u->user, u->host,
-				           type, to, prefix, line );
+				           type, to, prefix ? prefix : "", line );
 			}
 			line = s + 1;
 		}
