@@ -21,10 +21,6 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef _WIN32
 #include <sys/utsname.h>
 #endif
@@ -58,6 +54,8 @@
 #define DEFAULT_GROUPCHAT "conference.jabber.org"
 #define DEFAULT_PORT 5222
 #define DEFAULT_PORT_SSL 5223
+#define JABBER_PORT_MIN 5220
+#define JABBER_PORT_MAX 5229
 
 #define JABBER_GROUP "Friends"
 
@@ -545,11 +543,6 @@ static void gjab_connected_ssl(gpointer data, void *source, GaimInputCondition c
 	struct jabber_data *jd;
 	gjconn gjc;
 	
-	if (!g_slist_find(get_connections(), gc)) {
-		ssl_disconnect(source);
-		return;
-	}
-	
 	jd = gc->proto_data;
 	gjc = jd->gjc;
 	
@@ -558,6 +551,11 @@ static void gjab_connected_ssl(gpointer data, void *source, GaimInputCondition c
 		return;
 	}
 	
+	if (!g_slist_find(get_connections(), gc)) {
+		ssl_disconnect(source);
+		return;
+	}
+	
 	gjab_connected(data, gjc->fd, cond);
 }
 
@@ -565,16 +563,27 @@ static void gjab_start(gjconn gjc)
 {
 	struct aim_user *user;
 	int port = -1, ssl = 0;
+	char *server = NULL, *s;
 
 	if (!gjc || gjc->state != JCONN_STATE_OFF)
 		return;
 
 	user = GJ_GC(gjc)->user;
 	if (*user->proto_opt[0]) {
-		if (isdigit(user->proto_opt[0][0]))
-			sscanf(user->proto_opt[0], "%d", &port);
+		/* If there's a dot, assume there's a hostname in the beginning */
+		if (strchr(user->proto_opt[0], '.')) {
+			server = g_strdup(user->proto_opt[0]);
+			if ((s = strchr(server, ':')))
+				*s = 0;
+		}
 		
-		if (strstr(user->proto_opt[0], "ssl"))
+		/* After the hostname, there can be a port number */
+		s = strchr(user->proto_opt[0], ':');
+		if (s && isdigit(s[1]))
+			sscanf(s + 1, "%d", &port);
+		
+		/* And if there's the string ssl, the user wants an SSL-connection */
+		if (strstr(user->proto_opt[0], ":ssl") || g_strcasecmp(user->proto_opt[0], "ssl") == 0)
 			ssl = 1;
 	}
 	
@@ -582,6 +591,14 @@ static void gjab_start(gjconn gjc)
 		port = DEFAULT_PORT;
 	else if (port == -1 && ssl)
 		port = DEFAULT_PORT_SSL;
+	else if (port < JABBER_PORT_MIN || port > JABBER_PORT_MAX) {
+		serv_got_crap(GJ_GC(gjc), "For security reasons, the Jabber port number must be in the %d-%d range.", JABBER_PORT_MIN, JABBER_PORT_MAX);
+		STATE_EVT(JCONN_STATE_OFF)
+		return;
+	}
+	
+	if (server == NULL)
+		server = g_strdup(gjc->user->server);
 
 	gjc->parser = XML_ParserCreate(NULL);
 	XML_SetUserData(gjc->parser, (void *)gjc);
@@ -589,14 +606,16 @@ static void gjab_start(gjconn gjc)
 	XML_SetCharacterDataHandler(gjc->parser, charData);
 	
 	if (ssl) {
-		if ((gjc->ssl = ssl_connect(gjc->user->server, port, gjab_connected_ssl, GJ_GC(gjc))))
+		if ((gjc->ssl = ssl_connect(server, port, gjab_connected_ssl, GJ_GC(gjc))))
 			gjc->fd = ssl_getfd(gjc->ssl);
 		else
 			gjc->fd = -1;
 	} else {
-		gjc->fd = proxy_connect(gjc->user->server, port, gjab_connected, GJ_GC(gjc));
+		gjc->fd = proxy_connect(server, port, gjab_connected, GJ_GC(gjc));
 	}
 	
+	g_free(server);
+	
 	if (!user->gc || (gjc->fd < 0)) {
 		STATE_EVT(JCONN_STATE_OFF)
 		return;
@@ -745,10 +764,10 @@ static time_t iso8601_to_time(char *timestamp)
 
 static void jabber_handlemessage(gjconn gjc, jpacket p)
 {
-	xmlnode y, xmlns, subj, z;
+	xmlnode y, xmlns, z;
 	time_t time_sent = time(NULL);
 
-	char *from = NULL, *msg = NULL, *type = NULL, *topic = NULL;
+	char *from = NULL, *msg = NULL, *type = NULL;
 	char m[BUF_LONG * 2];
 
 	type = xmlnode_get_attrib(p->x, "type");
@@ -786,18 +805,7 @@ static void jabber_handlemessage(gjconn gjc, jpacket p)
 			return;
 
 		if (type && !g_strcasecmp(type, "jabber:x:conference")) {
-			char *room;
-			GList *m = NULL;
-			char **data;
-
-			room = xmlnode_get_attrib(xmlns, "jid");
-			data = g_strsplit(room, "@", 2);
-			m = g_list_append(m, g_strdup(data[0]));
-			m = g_list_append(m, g_strdup(data[1]));
-			m = g_list_append(m, g_strdup(gjc->user->user));
-			g_strfreev(data);
-
-			/* ** Bitlbee ** serv_got_chat_invite(GJ_GC(gjc), room, from, msg, m); */
+			/* do nothing */
 		} else if (msg) { /* whisper */
 			struct jabber_chat *jc;
 			g_snprintf(m, sizeof(m), "%s", msg);
@@ -805,22 +813,15 @@ static void jabber_handlemessage(gjconn gjc, jpacket p)
 				serv_got_chat_in(GJ_GC(gjc), jc->b->id, p->from->resource, 1, m, time_sent);
 			else {
 				int flags = 0;
-				/* ** Bitlbee **
-				if (xmlnode_get_tag(p->x, "gaim"))
-					flags = IM_FLAG_GAIMUSER;
-				if (find_conversation(jid_full(p->from)))
-					serv_got_im(GJ_GC(gjc), jid_full(p->from), m, flags, time_sent, -1);
-				else {
-				** End - Bitlbee ** */
-					if(p->from->user) {
-					    from = g_strdup_printf("%s@%s", p->from->user, p->from->server);
-					} else {
-					    /* server message? */
-					    from = g_strdup(p->from->server);
-					}
-					serv_got_im(GJ_GC(gjc), from, m, flags, time_sent, -1);
-					g_free(from);
-				/* ** Bitlbee ** } ** End - Bitlbee ** */
+				
+				if(p->from->user) {
+				    from = g_strdup_printf("%s@%s", p->from->user, p->from->server);
+				} else {
+				    /* server message? */
+				    from = g_strdup(p->from->server);
+				}
+				serv_got_im(GJ_GC(gjc), from, m, flags, time_sent, -1);
+				g_free(from);
 			}
 		}
 
@@ -835,75 +836,45 @@ static void jabber_handlemessage(gjconn gjc, jpacket p)
 			do_error_dialog(GJ_GC(gjc), msg, from);
 			g_free(from);
 		}
-	} else if (!g_strcasecmp(type, "groupchat")) {
-		struct jabber_chat *jc;
-		static int i = 0;
-
-		/*
-		if ((y = xmlnode_get_tag(p->x, "html"))) {
-			msg = xmlnode_get_data(y);
-		} else
-		*/
-		if ((y = xmlnode_get_tag(p->x, "body"))) {
-			msg = xmlnode_get_data(y);
-		}
-
-		msg = utf8_to_str(msg);
+	} else if (!g_strcasecmp(type, "headline")) {
+		char *subject, *body, *url;
 		
-		if ((subj = xmlnode_get_tag(p->x, "subject"))) {
-		   	topic = xmlnode_get_data(subj);
-		} 
-		topic = utf8_to_str(topic);
-
-		jc = find_existing_chat(GJ_GC(gjc), p->from);
-		if (!jc) {
-			/* we're not in this chat. are we supposed to be? */
-			if ((jc = find_pending_chat(GJ_GC(gjc), p->from)) != NULL) {
-				/* yes, we're supposed to be. so now we are. */
-				jc->b = serv_got_joined_chat(GJ_GC(gjc), i++, p->from->user);
-				jc->id = jc->b->id;
-				jc->state = JCS_ACTIVE;
-			} else {
-				/* no, we're not supposed to be. */
-				g_free(msg);
-				return;
+		y = xmlnode_get_tag( p->x, "body" );
+		body = y ? g_strdup( xmlnode_get_data( y ) ) : NULL;
+		
+		y = xmlnode_get_tag( p->x, "subject" );
+		subject = y ? g_strdup( xmlnode_get_data( y ) ) : NULL;
+		
+		url = NULL;
+		z = xmlnode_get_firstchild(p->x);
+		while( z )
+		{
+			char *xtype = xmlnode_get_attrib( z, "xmlns" );
+			
+			if( xtype && g_strcasecmp( xtype, "jabber:x:oob" ) == 0 &&
+			             ( y = xmlnode_get_tag( z, "url" ) ) )
+			{
+				url = g_strdup( xmlnode_get_data( y ) );
+				break;
 			}
+			
+			z = xmlnode_get_nextsibling( z );
 		}
-		if (p->from->resource) {
-			if (!y) {
-				if (!find_chat_buddy(jc->b, p->from->resource)) {
-					add_chat_buddy(jc->b, p->from->resource);
-				} else if ((y = xmlnode_get_tag(p->x, "status"))) {
-					char *buf;
-
-					buf = g_strdup_printf("%s@%s/%s",
-						p->from->user, p->from->server, p->from->resource);
-					jabber_track_away(gjc, p, buf, NULL);
-					g_free(buf);
-
-				}
-			} else if (jc->b && msg) {
-				char buf[8192];
-
-				if (topic) {
-					char tbuf[8192];
-					g_snprintf(tbuf, sizeof(tbuf), "%s", topic);
-				}
-				
-
-				g_snprintf(buf, sizeof(buf), "%s", msg);
-				serv_got_chat_in(GJ_GC(gjc), jc->b->id, p->from->resource, 0, buf, time_sent);
-			}
-		} else { /* message from the server */
-		   	if(jc->b && topic) {
-			   	char tbuf[8192];
-				g_snprintf(tbuf, sizeof(tbuf), "%s", topic);
-			}
-		}
-
-		g_free(msg);
-		g_free(topic);
+		
+		g_snprintf( m, BUF_LONG, "Subject: %s\nURL: %s\nMessage:\n%s", subject ? subject : "(none)",
+		                     url ? url : "(none)", body ? body : "(none)" );
 
+		if( p->from->user )
+			from = g_strdup_printf( "%s@%s", p->from->user, p->from->server );
+		else
+			from = g_strdup( p->from->server );
+		
+		serv_got_im( GJ_GC(gjc), from, m, 0, time_sent, -1 );
+		
+		g_free( from );
+		g_free( subject );
+		g_free( body );
+		g_free( url );
 	}
 }
 	   
@@ -1111,7 +1082,7 @@ static void jabber_handles10n(gjconn gjc, jpacket p)
 		 * A "subscribe to us" request was received - put up the approval dialog
 		 */
 		struct jabber_add_permit *jap = g_new0(struct jabber_add_permit, 1);
-		gchar *msg = g_strdup_printf(_("The user %s wants to add you to their buddy list."),
+		gchar *msg = g_strdup_printf(_("The user %s wants to add you to his/her buddy list."),
 				Jid);
 
 		jap->gjc = gjc;
@@ -1282,14 +1253,10 @@ static void jabber_handleauthresp(gjconn gjc, jpacket p)
 			}
 			gjab_auth(gjc);
 		} else {
-			account_online(GJ_GC(gjc));
-
-			if (bud_list_cache_exists(GJ_GC(gjc)))
-				do_import(GJ_GC(gjc), NULL);
-
-			((struct jabber_data *)GJ_GC(gjc)->proto_data)->did_import = TRUE;
-
 			gjab_reqroster(gjc);
+			account_online(GJ_GC(gjc));
+			
+			((struct jabber_data *)GJ_GC(gjc)->proto_data)->did_import = TRUE;
 		}
 	} else {
 		xmlnode xerr;
@@ -1303,7 +1270,7 @@ static void jabber_handleauthresp(gjconn gjc, jpacket p)
 			errmsg = xmlnode_get_data(xerr);
 			if (xmlnode_get_attrib(xerr, "code")) {
 				errcode = atoi(xmlnode_get_attrib(xerr, "code"));
-				g_snprintf(msg, sizeof(msg), "Error %d: %s", errcode, errmsg);
+				g_snprintf(msg, sizeof(msg), "Error %d: %s", errcode, errmsg ? errmsg : "Unknown error");
 			} else
 				g_snprintf(msg, sizeof(msg), "%s", errmsg);
 			hide_login_progress(GJ_GC(gjc), msg);
@@ -1893,11 +1860,7 @@ static void jabber_set_away(struct gaim_connection *gc, char *state, char *messa
 			y = xmlnode_insert_tag(x, "show");
 			xmlnode_insert_cdata(y, "away", -1);
 			y = xmlnode_insert_tag(x, "status");
-			{
-				char *utf8 = str_to_utf8(message);
-				xmlnode_insert_cdata(y, utf8, -1);
-				g_free(utf8);
-			}
+			xmlnode_insert_cdata(y, message, -1);
 			gc->away = "";
 		} else {
 			/* Gaim wants us to not be away */
@@ -2424,6 +2387,7 @@ void jabber_init(struct prpl *ret)
 	ret->buddy_free = jabber_buddy_free;
 	ret->alias_buddy = jabber_roster_update;
 	ret->group_buddy = jabber_group_change;
+	ret->cmp_buddynames = g_strcasecmp;
 
 	my_protocol = ret;
 }
