@@ -171,17 +171,17 @@ static GList *msn_away_states( struct gaim_connection *gc )
 	int i;
 	
 	for( i = 0; msn_away_state_list[i].number > -1; i ++ )
-		l = g_list_append( l, msn_away_state_list[i].name );
+		l = g_list_append( l, (void*) msn_away_state_list[i].name );
 	
 	return( l );
 }
 
 static char *msn_get_status_string( struct gaim_connection *gc, int number )
 {
-	struct msn_away_state *st = msn_away_state_by_number( number );
+	const struct msn_away_state *st = msn_away_state_by_number( number );
 	
 	if( st )
-		return( st->name );
+		return( (char*) st->name );
 	else
 		return( "" );
 }
@@ -190,7 +190,7 @@ static void msn_set_away( struct gaim_connection *gc, char *state, char *message
 {
 	char buf[1024];
 	struct msn_data *md = gc->proto_data;
-	struct msn_away_state *st;
+	const struct msn_away_state *st;
 	
 	if( strcmp( state, GAIM_AWAY_CUSTOM ) == 0 )
 		st = msn_away_state_by_name( "Away" );
@@ -368,7 +368,10 @@ static void msn_rem_deny( struct gaim_connection *gc, char *who )
 
 static int msn_send_typing( struct gaim_connection *gc, char *who, int typing )
 {
-	return( msn_send_im( gc, who, TYPING_NOTIFICATION_MESSAGE, strlen( TYPING_NOTIFICATION_MESSAGE ), 0 ) );
+	if( typing )
+		return( msn_send_im( gc, who, TYPING_NOTIFICATION_MESSAGE, strlen( TYPING_NOTIFICATION_MESSAGE ), 0 ) );
+	else
+		return( 1 );
 }
 
 void msn_init(struct prpl *ret)
@@ -394,6 +397,7 @@ void msn_init(struct prpl *ret)
 	ret->add_deny = msn_add_deny;
 	ret->rem_deny = msn_rem_deny;
 	ret->send_typing = msn_send_typing;
+	ret->cmp_buddynames = g_strcasecmp;
 
 	my_protocol = ret;
 }
