@@ -121,7 +121,7 @@ int proto_away( struct gaim_connection *gc, char *away )
 		{
 			gc->prpl->set_away( gc, s, away );
 			if( set_getint( gc->irc, "debug" ) )
-				irc_usermsg( gc->irc, "Setting away state for %s to %s", proto_name[gc->protocol], s );
+				serv_got_crap( gc, "Setting away state to %s", s );
 		}
 		else
 			gc->prpl->set_away( gc, GAIM_AWAY_CUSTOM, away );
@@ -162,44 +162,6 @@ static char *proto_away_alias_find( GList *gcm, char *away )
 	return( NULL );
 }
 
-/* Compare two handles for a specific protocol. For most protocols,
-   g_strcasecmp is okay, but for AIM, for example, it's not. This really
-   should be a compare function inside the PRPL module, but I do it this
-   way for now because I don't want to touch the Gaim code too much since
-   it's not going to be here for too long anymore. */
-int handle_cmp( char *a, char *b, int protocol )
-{
-	if( protocol == PROTO_TOC || protocol == PROTO_ICQ )
-	{
-		/* AIM, being teh evil, thinks it's cool that users can put
-		   random spaces in screennames. But "A B" and "AB" are
-		   equal. Hrmm, okay. */
-		while( 1 )
-		{
-			while( *a == ' ' ) a ++;
-			while( *b == ' ' ) b ++;
-			
-			if( *a && *b )
-			{
-				if( tolower( *a ) != tolower( *b ) )
-					return( 1 );
-			}
-			else if( *a || *b )
-				return( 1 );
-			else
-				return( 0 );
-			
-			a ++;
-			b ++;
-		}
-	}
-	else
-	{
-		return( g_strcasecmp( a, b ) );
-	}
-}
-
-
 /* multi.c */
 
 struct gaim_connection *new_gaim_conn( struct aim_user *user )
@@ -253,26 +215,27 @@ void destroy_gaim_conn( struct gaim_connection *gc )
 
 void set_login_progress( struct gaim_connection *gc, int step, char *msg )
 {
-	irc_usermsg( gc->irc, "%s - Logging in: %s", proto_name[gc->protocol], msg );
+	serv_got_crap( gc, "Logging in: %s", msg );
 }
 
 /* Errors *while* logging in */
 void hide_login_progress( struct gaim_connection *gc, char *msg )
 {
-	irc_usermsg( gc->irc, "%s - Login error: %s", proto_name[gc->protocol], msg );
+	serv_got_crap( gc, "Login error: %s", msg );
 }
 
 /* Errors *after* logging in */
 void hide_login_progress_error( struct gaim_connection *gc, char *msg )
 {
-	irc_usermsg( gc->irc, "%s - Logged out: %s", proto_name[gc->protocol], msg );
+	serv_got_crap( gc, "Logged out: %s", msg );
 }
 
 void serv_got_crap( struct gaim_connection *gc, char *format, ... )
 {
 	va_list params;
-	char text[1024], buf[1024];
+	char text[1024], buf[1024], acc_id[33];
 	char *msg;
+	account_t *a;
 	
 	va_start( params, format );
 	g_vsnprintf( text, sizeof( text ), format, params );
@@ -284,7 +247,22 @@ void serv_got_crap( struct gaim_connection *gc, char *format, ... )
 	else
 		msg = text;
 	
-	irc_usermsg( gc->irc, "%s - %s", proto_name[gc->protocol], msg );
+	if( ( g_strcasecmp( set_getstr( gc->irc, "strip_html" ), "always" ) == 0 ) ||
+	    ( ( gc->flags & OPT_CONN_HTML ) && set_getint( gc->irc, "strip_html" ) ) )
+		strip_html( msg );
+	
+	/* Try to find a different connection on the same protocol. */
+	for( a = gc->irc->accounts; a; a = a->next )
+		if( proto_prpl[a->protocol] == gc->prpl && a->gc != gc )
+			break;
+	
+	/* If we found one, add the screenname to the acc_id. */
+	if( a )
+		g_snprintf( acc_id, 32, "%s(%s)", proto_name[gc->protocol], gc->username );
+	else
+		g_snprintf( acc_id, 32, "%s", proto_name[gc->protocol] );
+	
+	irc_usermsg( gc->irc, "%s - %s", acc_id, msg );
 }
 
 static gboolean send_keepalive( gpointer d )
@@ -309,7 +287,7 @@ void account_online( struct gaim_connection *gc )
 	
 	u = user_find( gc->irc, gc->irc->nick );
 	
-	irc_usermsg( gc->irc, "%s - Logged in", proto_name[gc->protocol] );
+	serv_got_crap( gc, "Logged in" );
 	
 	gc->keepalive = g_timeout_add( 60000, send_keepalive, gc );
 	gc->flags |= OPT_LOGGED_IN;
@@ -323,7 +301,7 @@ void account_online( struct gaim_connection *gc )
 				break;
 		
 		if( u == NULL )
-			irc_usermsg( gc->irc, "\x02""***\x02"" BitlBee now supports ICQ server-side contact lists. "
+			serv_got_crap( gc, "\x02""***\x02"" BitlBee now supports ICQ server-side contact lists. "
 			                      "See \x02""help import_buddies\x02"" for more information." );
 	}
 }
@@ -356,7 +334,7 @@ void signoff( struct gaim_connection *gc )
 	user_t *t, *u = irc->users;
 	account_t *a;
 	
-	irc_usermsg( gc->irc, "%s - Signing off..", proto_name[gc->protocol] );
+	serv_got_crap( gc, "Signing off.." );
 
 	gaim_input_remove( gc->keepalive );
 	gc->keepalive = 0;
@@ -388,7 +366,7 @@ void signoff( struct gaim_connection *gc )
 	else if( !gc->wants_to_die && set_getint( irc, "auto_reconnect" ) )
 	{
 		int delay = set_getint( irc, "auto_reconnect_delay" );
-		irc_usermsg( gc->irc, "%s - Reconnecting in %d seconds..", proto_name[gc->protocol], delay);
+		serv_got_crap( gc, "Reconnecting in %d seconds..", delay );
 		
 		a->reconnect = 1;
 		g_timeout_add( delay * 1000, auto_reconnect, a );
@@ -402,7 +380,14 @@ void signoff( struct gaim_connection *gc )
 
 void do_error_dialog( struct gaim_connection *gc, char *msg, char *title )
 {
-	irc_usermsg( gc->irc, "%s - Error: %s", title, msg );
+	if( msg && title )
+		serv_got_crap( gc, "Error: %s: %s", title, msg );
+	else if( msg )
+		serv_got_crap( gc, "Error: %s", msg );
+	else if( title )
+		serv_got_crap( gc, "Error: %s", title );
+	else
+		serv_got_crap( gc, "Error" );
 }
 
 void do_ask_dialog( struct gaim_connection *gc, char *msg, void *data, void *doit, void *dont )
@@ -431,12 +416,12 @@ void add_buddy( struct gaim_connection *gc, char *group, char *handle, char *rea
 	irc_t *irc = gc->irc;
 	
 	if( set_getint( irc, "debug" ) && 0 ) /* This message is too useless */
-		irc_usermsg( irc, "Receiving user add from protocol: %s", handle );
+		serv_got_crap( gc, "Receiving user add from handle: %s", handle );
 	
 	if( user_findhandle( gc, handle ) )
 	{
 		if( set_getint( irc, "debug" ) )
-			irc_usermsg( irc, "User already exists, ignoring add request: %s", handle );
+			serv_got_crap( gc, "User already exists, ignoring add request: %s", handle );
 		
 		return;
 		
@@ -529,7 +514,7 @@ void serv_buddy_rename( struct gaim_connection *gc, char *handle, char *realname
 		u->realname = g_strdup( name );
 		
 		if( ( gc->flags & OPT_LOGGED_IN ) && set_getint( gc->irc, "display_namechanges" ) )
-			irc_usermsg( gc->irc, "User `%s' changed name to `%s'", u->nick, u->realname );
+			serv_got_crap( gc, "User `%s' changed name to `%s'", u->nick, u->realname );
 	}
 }
 
@@ -562,8 +547,8 @@ void serv_got_update( struct gaim_connection *gc, char *handle, int loggedin, in
 		{
 			if( set_getint( gc->irc, "debug" ) || g_strcasecmp( set_getstr( gc->irc, "handle_unknown" ), "ignore" ) != 0 )
 			{
-				irc_usermsg( gc->irc, "serv_got_update() for unknown %s handle %s:", proto_name[gc->protocol], handle );
-				irc_usermsg( gc->irc, "loggedin = %d, type = %d", loggedin, type );
+				serv_got_crap( gc, "serv_got_update() for handle %s:", handle );
+				serv_got_crap( gc, "loggedin = %d, type = %d", loggedin, type );
 			}
 			
 			return;
@@ -643,7 +628,7 @@ void serv_got_im( struct gaim_connection *gc, char *handle, char *msg, guint32 f
 		if( g_strcasecmp( h, "ignore" ) == 0 )
 		{
 			if( set_getint( irc, "debug" ) )
-				irc_usermsg( irc, "Ignoring message from unknown %s handle %s", proto_name[gc->protocol], handle );
+				serv_got_crap( gc, "Ignoring message from unknown handle %s", handle );
 			
 			return;
 		}
@@ -665,57 +650,71 @@ void serv_got_im( struct gaim_connection *gc, char *handle, char *msg, guint32 f
 		}
 		else
 		{
-			irc_usermsg( irc, "Message from unknown %s handle %s:", proto_name[gc->protocol], handle );
+			serv_got_crap( gc, "Message from unknown handle %s:", handle );
 			u = user_find( irc, irc->mynick );
 		}
 	}
 	
-	if( g_strcasecmp( set_getstr( irc, "html" ), "strip" ) == 0 )
+	if( ( g_strcasecmp( set_getstr( gc->irc, "strip_html" ), "always" ) == 0 ) ||
+	    ( ( gc->flags & OPT_CONN_HTML ) && set_getint( gc->irc, "strip_html" ) ) )
 		strip_html( msg );
 
 	if( g_strncasecmp( set_getstr( irc, "charset" ), "none", 4 ) != 0 &&
 	    do_iconv( "UTF-8", set_getstr( irc, "charset" ), msg, buf, 0, 8192 ) != -1 )
 		msg = buf;
 	
-	while( strlen( msg ) > 450 )
+	while( strlen( msg ) > 425 )
 	{
 		char tmp, *nl;
 		
-		tmp = msg[450];
-		msg[450] = 0;
+		tmp = msg[425];
+		msg[425] = 0;
 		
-		/* If there's a newline in this string, split up there so we're not
-		   going to split up lines. If there isn't a newline, well, too bad. */
-		if( ( nl = strrchr( msg, '\n' ) ) )
+		/* If there's a newline/space in this string, split up there,
+		   looks a bit prettier. */
+		if( ( nl = strrchr( msg, '\n' ) ) || ( nl = strrchr( msg, ' ' ) ) )
+		{
+			msg[425] = tmp;
+			tmp = *nl;
 			*nl = 0;
+		}
 		
 		irc_msgfrom( irc, u->nick, msg );
 		
-		msg[450] = tmp;
-		
 		/* Move on. */
 		if( nl )
 		{
-			*nl = '\n';
+			*nl = tmp;
 			msg = nl + 1;
 		}
 		else
 		{
-			msg += 450;
+			msg[425] = tmp;
+			msg += 425;
 		}
 	}
 	irc_msgfrom( irc, u->nick, msg );
 }
 
-void serv_got_typing( struct gaim_connection *gc, char *handle, int timeout )
+void serv_got_typing( struct gaim_connection *gc, char *handle, int timeout, int type )
 {
 	user_t *u;
 	
 	if( !set_getint( gc->irc, "typing_notice" ) )
 		return;
 	
-	if( ( u = user_findhandle( gc, handle ) ) )
-		irc_noticefrom( gc->irc, u->nick, "* Typing a message *" );
+	if( ( u = user_findhandle( gc, handle ) ) ) {
+		/* If type is:
+		 * 0: user has stopped typing
+		 * 1: user is actively typing
+		 * 2: user has entered text, but is not actively typing
+		 */
+		if (type == 0 || type == 1 || type == 2) {
+			char buf[256]; 
+			g_snprintf(buf, 256, "\1TYPING %d\1", type); 
+			irc_privmsg( gc->irc, u, "PRIVMSG", gc->irc->nick, NULL, buf );
+		}
+	}
 }
 
 void serv_got_chat_left( struct gaim_connection *gc, int id )
@@ -724,7 +723,7 @@ void serv_got_chat_left( struct gaim_connection *gc, int id )
 	GList *ir;
 	
 	if( set_getint( gc->irc, "debug" ) )
-		irc_usermsg( gc->irc, "You were removed from conversation %d", (int) id );
+		serv_got_crap( gc, "You were removed from conversation %d", (int) id );
 	
 	for( c = gc->conversations; c && c->id != id; c = (l=c)->next );
 	
@@ -769,7 +768,8 @@ void serv_got_chat_in( struct gaim_connection *gc, int id, char *who, int whispe
 	u = user_findhandle( gc, who );
 	for( c = gc->conversations; c && c->id != id; c = c->next );
 	
-	if( g_strcasecmp( set_getstr( gc->irc, "html" ), "strip" ) == 0 )
+	if( ( g_strcasecmp( set_getstr( gc->irc, "strip_html" ), "always" ) == 0 ) ||
+	    ( ( gc->flags & OPT_CONN_HTML ) && set_getint( gc->irc, "strip_html" ) ) )
 		strip_html( msg );
 	
 	if( g_strncasecmp( set_getstr( gc->irc, "charset" ), "none", 4 ) != 0 &&
@@ -779,7 +779,7 @@ void serv_got_chat_in( struct gaim_connection *gc, int id, char *who, int whispe
 	if( c && u )
 		irc_privmsg( gc->irc, u, "PRIVMSG", c->channel, "", msg );
 	else
-		irc_usermsg( gc->irc, "Message from/to conversation %s@%d (unknown conv/user): %s", who, id, msg );
+		serv_got_crap( gc, "Message from/to conversation %s@%d (unknown conv/user): %s", who, id, msg );
 }
 
 struct conversation *serv_got_joined_chat( struct gaim_connection *gc, int id, char *handle )
@@ -802,12 +802,12 @@ struct conversation *serv_got_joined_chat( struct gaim_connection *gc, int id, c
 	c->title = g_strdup( handle );
 	
 	s = g_new( char, 16 );
-	sprintf( s, "#chat_%03d", gc->irc->c_id++ );
+	sprintf( s, "&chat_%03d", gc->irc->c_id++ );
 	c->channel = g_strdup( s );
 	g_free( s );
 	
 	if( set_getint( gc->irc, "debug" ) )
-		irc_usermsg( gc->irc, "Creating new conversation: (id=%d,handle=%s)", id, handle );
+		serv_got_crap( gc, "Creating new conversation: (id=%d,handle=%s)", id, handle );
 	
 	return( c );
 }
@@ -826,10 +826,10 @@ void add_chat_buddy( struct conversation *b, char *handle )
 	int me = 0;
 	
 	if( set_getint( b->gc->irc, "debug" ) )
-		irc_usermsg( b->gc->irc, "User %s added to conversation %d", handle, b->id );
+		serv_got_crap( b->gc, "User %s added to conversation %d", handle, b->id );
 	
 	/* It might be yourself! */
-	if( g_strcasecmp( handle, b->gc->user->username ) == 0 )
+	if( b->gc->prpl->cmp_buddynames( handle, b->gc->user->username ) == 0 )
 	{
 		u = user_find( b->gc->irc, b->gc->irc->nick );
 		if( !b->joined )
@@ -860,7 +860,7 @@ void remove_chat_buddy( struct conversation *b, char *handle, char *reason )
 	int me = 0;
 	
 	if( set_getint( b->gc->irc, "debug" ) )
-		irc_usermsg( b->gc->irc, "User %s removed from conversation %d (%s)", handle, b->id, reason ? reason : "" );
+		serv_got_crap( b->gc, "User %s removed from conversation %d (%s)", handle, b->id, reason ? reason : "" );
 	
 	/* It might be yourself! */
 	if( g_strcasecmp( handle, b->gc->user->username ) == 0 )
@@ -993,15 +993,26 @@ char *set_eval_away_devoice( irc_t *irc, set_t *set, char *value )
 	return( set_eval_bool( irc, set, value ) );
 }
 
-int serv_send_im( irc_t *irc, user_t *u, char *msg )
+int serv_send_im( irc_t *irc, user_t *u, char *msg, int flags )
 {
 	char buf[8192];
 	
 	if( g_strncasecmp( set_getstr( irc, "charset" ), "none", 4 ) != 0 &&
 	    do_iconv( set_getstr( irc, "charset" ), "UTF-8", msg, buf, 0, 8192 ) != -1 )
 		msg = buf;
+
+	if( ( u->gc->flags & OPT_CONN_HTML ) && ( g_strncasecmp( msg, "<html>", 6 ) != 0 ) )
+	{
+		char *html;
+		
+		html = escape_html( msg );
+		strncpy( buf, html, 8192 );
+		g_free( html );
+		
+		msg = buf;
+	}
 	
-	return( ((struct gaim_connection *)u->gc)->prpl->send_im( u->gc, u->handle, msg, strlen( msg ), 0 ) );
+	return( ((struct gaim_connection *)u->gc)->prpl->send_im( u->gc, u->handle, msg, strlen( msg ), flags ) );
 }
 
 int serv_send_chat( irc_t *irc, struct gaim_connection *gc, int id, char *msg )
@@ -1011,6 +1022,12 @@ int serv_send_chat( irc_t *irc, struct gaim_connection *gc, int id, char *msg )
 	if( g_strncasecmp( set_getstr( irc, "charset" ), "none", 4 ) != 0 &&
 	    do_iconv( set_getstr( irc, "charset" ), "UTF-8", msg, buf, 0, 8192 ) != -1 )
 		msg = buf;
+
+	if( gc->flags & OPT_CONN_HTML) {
+		char * html = escape_html(msg);
+		strncpy(buf, html, 8192);
+		g_free(html);
+	}
 	
 	return( gc->prpl->chat_send( gc, id, msg ) );
 }
