@@ -33,8 +33,15 @@
 #include "bitlbee.h"
 #include "proxy.h"
 
-#define FAIM_NEED_TLV
 #include "aim.h"
+#include "icq.h"
+#include "bos.h"
+#include "ssi.h"
+#include "im.h"
+#include "info.h"
+#include "buddylist.h"
+#include "chat.h"
+#include "chatnav.h"
 
 /* constants to identify proto_opts */
 #define USEROPT_AUTH      0
@@ -54,7 +61,7 @@
 /* Don't know if support for UTF8 is really working. For now it's UTF16 here.
    static int gaim_caps = AIM_CAPS_UTF8; */
 
-static int gaim_caps = AIM_CAPS_INTEROP | AIM_CAPS_ICHAT;
+static int gaim_caps = AIM_CAPS_INTEROP | AIM_CAPS_ICHAT | AIM_CAPS_ICQSERVERRELAY | AIM_CAPS_CHAT;
 static guint8 gaim_features[] = {0x01, 0x01, 0x01, 0x02};
 
 struct oscar_data {
@@ -148,7 +155,6 @@ static char *extract_name(const char *name) {
 	return tmp;
 }
 
-#if 0
 static struct chat_connection *find_oscar_chat(struct gaim_connection *gc, int id) {
 	GSList *g = ((struct oscar_data *)gc->proto_data)->oscar_chats;
 	struct chat_connection *c = NULL;
@@ -163,7 +169,7 @@ static struct chat_connection *find_oscar_chat(struct gaim_connection *gc, int i
 
 	return c;
 }
-#endif
+
 
 static struct chat_connection *find_oscar_chat_by_conn(struct gaim_connection *gc,
 							aim_conn_t *conn) {
@@ -213,8 +219,9 @@ static int gaim_offlinemsgdone   (aim_session_t *, aim_frame_t *, ...);
 static int gaim_ssi_parserights  (aim_session_t *, aim_frame_t *, ...);
 static int gaim_ssi_parselist    (aim_session_t *, aim_frame_t *, ...);
 static int gaim_ssi_parseack     (aim_session_t *, aim_frame_t *, ...);
-
+static int gaim_parsemtn         (aim_session_t *, aim_frame_t *, ...);
 static int gaim_icqinfo          (aim_session_t *, aim_frame_t *, ...);
+static int gaim_parseaiminfo     (aim_session_t *, aim_frame_t *, ...);
 
 static char *msgerrreason[] = {
 	"Invalid error",
@@ -266,53 +273,47 @@ static void oscar_callback(gpointer data, gint source,
 	odata = (struct oscar_data *)gc->proto_data;
 
 	if (condition & GAIM_INPUT_READ) {
-		if (conn->type == AIM_CONN_TYPE_RENDEZVOUS_OUT) {
-			if (aim_handlerendconnect(odata->sess, conn) < 0) {
-				aim_conn_kill(odata->sess, &conn);
-			}
+		if (aim_get_command(odata->sess, conn) >= 0) {
+			aim_rxdispatch(odata->sess);
+                               if (odata->killme)
+                                       signoff(gc);
 		} else {
-			if (aim_get_command(odata->sess, conn) >= 0) {
-				aim_rxdispatch(odata->sess);
-                                if (odata->killme)
-                                        signoff(gc);
-			} else {
-				if ((conn->type == AIM_CONN_TYPE_BOS) ||
-					   !(aim_getconn_type(odata->sess, AIM_CONN_TYPE_BOS))) {
-					hide_login_progress_error(gc, _("Disconnected."));
-					signoff(gc);
-				} else if (conn->type == AIM_CONN_TYPE_CHAT) {
-					struct chat_connection *c = find_oscar_chat_by_conn(gc, conn);
-					char buf[BUF_LONG];
-					c->conn = NULL;
-					if (c->inpa > 0)
-						gaim_input_remove(c->inpa);
-					c->inpa = 0;
-					c->fd = -1;
-					aim_conn_kill(odata->sess, &conn);
-					sprintf(buf, _("You have been disconnected from chat room %s."), c->name);
-					do_error_dialog(sess->aux_data, buf, _("Chat Error!"));
-				} else if (conn->type == AIM_CONN_TYPE_CHATNAV) {
-					if (odata->cnpa > 0)
-						gaim_input_remove(odata->cnpa);
-					odata->cnpa = 0;
-					while (odata->create_rooms) {
-						struct create_room *cr = odata->create_rooms->data;
-						g_free(cr->name);
-						odata->create_rooms =
-							g_slist_remove(odata->create_rooms, cr);
-						g_free(cr);
-						do_error_dialog(sess->aux_data, _("Chat is currently unavailable"),
-								_("Gaim - Chat"));
-					}
-					aim_conn_kill(odata->sess, &conn);
-				} else if (conn->type == AIM_CONN_TYPE_AUTH) {
-					if (odata->paspa > 0)
-						gaim_input_remove(odata->paspa);
-					odata->paspa = 0;
-					aim_conn_kill(odata->sess, &conn);
-				} else {
-					aim_conn_kill(odata->sess, &conn);
+			if ((conn->type == AIM_CONN_TYPE_BOS) ||
+				   !(aim_getconn_type(odata->sess, AIM_CONN_TYPE_BOS))) {
+				hide_login_progress_error(gc, _("Disconnected."));
+				signoff(gc);
+			} else if (conn->type == AIM_CONN_TYPE_CHAT) {
+				struct chat_connection *c = find_oscar_chat_by_conn(gc, conn);
+				char buf[BUF_LONG];
+				c->conn = NULL;
+				if (c->inpa > 0)
+					gaim_input_remove(c->inpa);
+				c->inpa = 0;
+				c->fd = -1;
+				aim_conn_kill(odata->sess, &conn);
+				sprintf(buf, _("You have been disconnected from chat room %s."), c->name);
+				do_error_dialog(sess->aux_data, buf, _("Chat Error!"));
+			} else if (conn->type == AIM_CONN_TYPE_CHATNAV) {
+				if (odata->cnpa > 0)
+					gaim_input_remove(odata->cnpa);
+				odata->cnpa = 0;
+				while (odata->create_rooms) {
+					struct create_room *cr = odata->create_rooms->data;
+					g_free(cr->name);
+					odata->create_rooms =
+						g_slist_remove(odata->create_rooms, cr);
+					g_free(cr);
+					do_error_dialog(sess->aux_data, _("Chat is currently unavailable"),
+							_("Gaim - Chat"));
 				}
+				aim_conn_kill(odata->sess, &conn);
+			} else if (conn->type == AIM_CONN_TYPE_AUTH) {
+				if (odata->paspa > 0)
+					gaim_input_remove(odata->paspa);
+				odata->paspa = 0;
+				aim_conn_kill(odata->sess, &conn);
+			} else {
+				aim_conn_kill(odata->sess, &conn);
 			}
 		}
 	}
@@ -392,9 +393,9 @@ static void oscar_login(struct aim_user *user) {
 
 	conn->status |= AIM_CONN_STATUS_INPROGRESS;
 	conn->fd = proxy_connect(user->proto_opt[USEROPT_AUTH][0] ?
-					user->proto_opt[USEROPT_AUTH] : FAIM_LOGIN_SERVER,
+					user->proto_opt[USEROPT_AUTH] : AIM_DEFAULT_LOGIN_SERVER,
 				 user->proto_opt[USEROPT_AUTHPORT][0] ?
-					atoi(user->proto_opt[USEROPT_AUTHPORT]) : FAIM_LOGIN_PORT,
+					atoi(user->proto_opt[USEROPT_AUTHPORT]) : AIM_LOGIN_PORT,
 				 oscar_login_connect, gc);
 	if (conn->fd < 0) {
 		hide_login_progress(gc, _("Couldn't connect to host"));
@@ -479,7 +480,7 @@ static int gaim_parse_auth_resp(aim_session_t *sess, aim_frame_t *fr, ...) {
         struct oscar_data *od = gc->proto_data;
 	user = gc->user;
 	port = user->proto_opt[USEROPT_AUTHPORT][0] ?
-		atoi(user->proto_opt[USEROPT_AUTHPORT]) : FAIM_LOGIN_PORT,
+		atoi(user->proto_opt[USEROPT_AUTHPORT]) : AIM_LOGIN_PORT,
 
 	va_start(ap, fr);
 	info = va_arg(ap, struct aim_authresp_info *);
@@ -548,6 +549,8 @@ static int gaim_parse_auth_resp(aim_session_t *sess, aim_frame_t *fr, ...) {
 	aim_conn_addhandler(sess, bosconn, AIM_CB_FAM_SSI, AIM_CB_SSI_RIGHTSINFO, gaim_ssi_parserights, 0);
 	aim_conn_addhandler(sess, bosconn, AIM_CB_FAM_SSI, AIM_CB_SSI_LIST, gaim_ssi_parselist, 0);
 	aim_conn_addhandler(sess, bosconn, AIM_CB_FAM_SSI, AIM_CB_SSI_SRVACK, gaim_ssi_parseack, 0);
+	aim_conn_addhandler(sess, bosconn, AIM_CB_FAM_LOC, AIM_CB_LOC_USERINFO, gaim_parseaiminfo, 0);
+	aim_conn_addhandler(sess, bosconn, AIM_CB_FAM_MSG, AIM_CB_MSG_MTN, gaim_parsemtn, 0);
 
 	((struct oscar_data *)gc->proto_data)->conn = bosconn;
 	for (i = 0; i < (int)strlen(info->bosip); i++) {
@@ -854,7 +857,7 @@ static int gaim_handle_redirect(aim_session_t *sess, aim_frame_t *fr, ...) {
 	int port;
 
 	port = user->proto_opt[USEROPT_AUTHPORT][0] ?
-		atoi(user->proto_opt[USEROPT_AUTHPORT]) : FAIM_LOGIN_PORT,
+		atoi(user->proto_opt[USEROPT_AUTHPORT]) : AIM_LOGIN_PORT,
 
 	va_start(ap, fr);
 	redir = va_arg(ap, struct aim_redirect_data *);
@@ -1071,30 +1074,38 @@ static int incomingim_chan1(aim_session_t *sess, aim_conn_t *conn, aim_userinfo_
 	return 1;
 }
 
+void oscar_accept_chat(gpointer w, struct aim_chat_invitation * inv);
+void oscar_reject_chat(gpointer w, struct aim_chat_invitation * inv);
+	
 static int incomingim_chan2(aim_session_t *sess, aim_conn_t *conn, aim_userinfo_t *userinfo, struct aim_incomingim_ch2_args *args) {
-#if 0
 	struct gaim_connection *gc = sess->aux_data;
-#endif
 
 	if (args->status != AIM_RENDEZVOUS_PROPOSE)
 		return 1;
-#if 0
+
 	if (args->reqclass & AIM_CAPS_CHAT) {
 		char *name = extract_name(args->info.chat.roominfo.name);
 		int *exch = g_new0(int, 1);
 		GList *m = NULL;
+		char txt[1024];
+		struct aim_chat_invitation * inv = g_new0(struct aim_chat_invitation, 1);
+
 		m = g_list_append(m, g_strdup(name ? name : args->info.chat.roominfo.name));
 		*exch = args->info.chat.roominfo.exchange;
 		m = g_list_append(m, exch);
-		serv_got_chat_invite(gc,
-				     name ? name : args->info.chat.roominfo.name,
-				     userinfo->sn,
-				     (char *)args->msg,
-				     m);
+
+		g_snprintf( txt, 1024, "Got an invitation to chatroom %s from %s: %s", name, userinfo->sn, args->msg );
+
+		inv->gc = gc;
+		inv->exchange = *exch;
+		inv->name = g_strdup(name);
+		
+		do_ask_dialog( gc, txt, inv, oscar_accept_chat, oscar_reject_chat);
+	
 		if (name)
 			g_free(name);
 	}
-#endif
+
 	return 1;
 }
 
@@ -1210,6 +1221,10 @@ static int gaim_parse_incoming_im(aim_session_t *sess, aim_frame_t *fr, ...) {
 	channel = va_arg(ap, int);
 	userinfo = va_arg(ap, aim_userinfo_t *);
 
+    if (set_getint(sess->aux_data, "debug")) {
+        serv_got_crap(sess->aux_data, "channel %i called", channel);
+    }
+
 	switch (channel) {
 		case 1: { /* standard message */
 			struct aim_incomingim_ch1_args *args;
@@ -1367,6 +1382,7 @@ static int gaim_parse_locerr(aim_session_t *sess, aim_frame_t *fr, ...) {
 			(reason < msgerrreasonlen) ? msgerrreason[reason] : _("Reason unknown"));
 	do_error_dialog(sess->aux_data, buf, _("Gaim - Error"));
 
+
 	return 1;
 }
 
@@ -1663,6 +1679,7 @@ static int gaim_icbm_param_info(aim_session_t *sess, aim_frame_t *fr, ...) {
 	va_end(ap);
 
 	/* Maybe senderwarn and recverwarn should be user preferences... */
+	params->flags = 0x0000000b;
 	params->maxmsglen = 8000;
 	params->minmsginterval = 0;
 
@@ -1869,11 +1886,10 @@ static void oscar_get_info(struct gaim_connection *g, char *name) {
 	struct oscar_data *odata = (struct oscar_data *)g->proto_data;
 	if (odata->icq)
 		aim_icq_getallinfo(odata->sess, name);
-	else
-		/* people want the away message on the top, so we get the away message
-		 * first and then get the regular info, since it's too difficult to
-		 * insert in the middle. i hate people. */
+	else {
 		aim_getinfo(odata->sess, odata->conn, name, AIM_GETINFO_AWAYMESSAGE);
+		aim_getinfo(odata->sess, odata->conn, name, AIM_GETINFO_GENERALINFO);
+	}
 }
 
 static void oscar_get_away(struct gaim_connection *g, char *who) {
@@ -1885,15 +1901,25 @@ static void oscar_get_away(struct gaim_connection *g, char *who) {
 				if (budlight->caps & AIM_CAPS_ICQSERVERRELAY)
 					aim_send_im_ch2_geticqmessage(odata->sess, who, (budlight->uc & 0xff80) >> 7);
 	} else
-		aim_getinfo(odata->sess, odata->conn, who, AIM_GETINFO_GENERALINFO);
+		aim_getinfo(odata->sess, odata->conn, who, AIM_GETINFO_AWAYMESSAGE);
 }
 
-static void oscar_set_away_aim(struct gaim_connection *gc, struct oscar_data *od, const char *message)
+static void oscar_set_away_aim(struct gaim_connection *gc, struct oscar_data *od, const char *state, const char *message)
 {
 
+	if (!g_strcasecmp(state, _("Visible"))) {
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_NORMAL);
+		return;
+	} else if (!g_strcasecmp(state, _("Invisible"))) {
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_INVISIBLE);
+		return;
+	} /* else... */
+
 	if (od->rights.maxawaymsglen == 0)
 		do_error_dialog(gc, "oscar_set_away_aim called before locate rights received", "Protocol Error");
 
+	aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_NORMAL);
+
 	if (gc->away)
 		g_free(gc->away);
 	gc->away = NULL;
@@ -1921,36 +1947,56 @@ static void oscar_set_away_aim(struct gaim_connection *gc, struct oscar_data *od
 
 static void oscar_set_away_icq(struct gaim_connection *gc, struct oscar_data *od, const char *state, const char *message)
 {
+    const char *msg = NULL;
+	gboolean no_message = FALSE;
 
-	if (gc->away)
+	/* clean old states */
+    if (gc->away) {
+		g_free(gc->away);
 		gc->away = NULL;
+    }
+	od->sess->aim_icq_state = 0;
 
-	if (!g_strcasecmp(state, "Online"))
-		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_NORMAL, "");
-	else if (!g_strcasecmp(state, "Away")) {
-		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_AWAY, "");
-		gc->away = "";
+	/* if no message, then use an empty message */
+    if (message) {
+        msg = message;
+    } else {
+        msg = "";
+		no_message = TRUE;
+    }
+
+	if (!g_strcasecmp(state, "Online")) {
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_NORMAL);
+	} else if (!g_strcasecmp(state, "Away")) {
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_AWAY);
+        gc->away = g_strdup(msg);
+		od->sess->aim_icq_state = AIM_MTYPE_AUTOAWAY;
 	} else if (!g_strcasecmp(state, "Do Not Disturb")) {
-		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_AWAY | AIM_ICQ_STATE_DND | AIM_ICQ_STATE_BUSY, "");
-		gc->away = "";
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_AWAY | AIM_ICQ_STATE_DND | AIM_ICQ_STATE_BUSY);
+        gc->away = g_strdup(msg);
+		od->sess->aim_icq_state = AIM_MTYPE_AUTODND;
 	} else if (!g_strcasecmp(state, "Not Available")) {
-		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_OUT | AIM_ICQ_STATE_AWAY, "");
-		gc->away = "";
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_OUT | AIM_ICQ_STATE_AWAY);
+        gc->away = g_strdup(msg);
+		od->sess->aim_icq_state = AIM_MTYPE_AUTONA;
 	} else if (!g_strcasecmp(state, "Occupied")) {
-		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_AWAY | AIM_ICQ_STATE_BUSY, "");
-		gc->away = "";
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_AWAY | AIM_ICQ_STATE_BUSY);
+        gc->away = g_strdup(msg);
+		od->sess->aim_icq_state = AIM_MTYPE_AUTOBUSY;
 	} else if (!g_strcasecmp(state, "Free For Chat")) {
-		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_CHAT, "");
-		gc->away = "";
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_CHAT);
+        gc->away = g_strdup(msg);
+		od->sess->aim_icq_state = AIM_MTYPE_AUTOFFC;
 	} else if (!g_strcasecmp(state, "Invisible")) {
-		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_INVISIBLE, "");
-		gc->away = "";
+		aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_INVISIBLE);
+        gc->away = g_strdup(msg);
 	} else if (!g_strcasecmp(state, GAIM_AWAY_CUSTOM)) {
-	 	if (message) {
-			aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_OUT | AIM_ICQ_STATE_AWAY, "");
-			gc->away = "";
+	 	if (no_message) {
+			aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_NORMAL);
 		} else {
-			aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_NORMAL, "");
+			aim_setextstatus(od->sess, od->conn, AIM_ICQ_STATE_AWAY);
+            gc->away = g_strdup(msg);
+			od->sess->aim_icq_state = AIM_MTYPE_AUTOAWAY;
 		}
 	}
 
@@ -1961,10 +2007,9 @@ static void oscar_set_away(struct gaim_connection *gc, char *state, char *messag
 {
 	struct oscar_data *od = (struct oscar_data *)gc->proto_data;
 
+    oscar_set_away_aim(gc, od, state, message);
 	if (od->icq)
 		oscar_set_away_icq(gc, od, state, message);
-	else
-		oscar_set_away_aim(gc, od, message);
 
 	return;
 }
@@ -2286,6 +2331,141 @@ static int gaim_icqinfo(aim_session_t *sess, aim_frame_t *fr, ...)
 
 }
 
+static char *oscar_encoding_extract(const char *encoding)
+{
+	char *ret = NULL;
+	char *begin, *end;
+
+	g_return_val_if_fail(encoding != NULL, NULL);
+
+	/* Make sure encoding begins with charset= */
+	if (strncmp(encoding, "text/plain; charset=", 20) &&
+		strncmp(encoding, "text/aolrtf; charset=", 21) &&
+		strncmp(encoding, "text/x-aolrtf; charset=", 23))
+	{
+		return NULL;
+	}
+
+	begin = strchr(encoding, '"');
+	end = strrchr(encoding, '"');
+
+	if ((begin == NULL) || (end == NULL) || (begin >= end))
+		return NULL;
+
+	ret = g_strndup(begin+1, (end-1) - begin);
+
+	return ret;
+}
+
+static char *oscar_encoding_to_utf8(char *encoding, char *text, int textlen)
+{
+	char *utf8 = g_new0(char, 8192);
+
+	if ((encoding == NULL) || encoding[0] == '\0') {
+		/*		gaim_debug_info("oscar", "Empty encoding, assuming UTF-8\n");*/
+	} else if (!g_strcasecmp(encoding, "iso-8859-1")) {
+		do_iconv("iso-8859-1", "UTF-8", text, utf8, textlen, 8192);
+	} else if (!g_strcasecmp(encoding, "ISO-8859-1-Windows-3.1-Latin-1")) {
+		do_iconv("Windows-1252", "UTF-8", text, utf8, textlen, 8192);
+	} else if (!g_strcasecmp(encoding, "unicode-2-0")) {
+		do_iconv("UCS-2BE", "UTF-8", text, utf8, textlen, 8192);
+	} else if (g_strcasecmp(encoding, "us-ascii") && strcmp(encoding, "utf-8")) {
+		/*		gaim_debug_warning("oscar", "Unrecognized character encoding \"%s\", "
+		  "attempting to convert to UTF-8 anyway\n", encoding);*/
+		do_iconv(encoding, "UTF-8", text, utf8, textlen, 8192);
+	}
+
+	/*
+	 * If utf8 is still NULL then either the encoding is us-ascii/utf-8 or
+	 * we have been unable to convert the text to utf-8 from the encoding
+	 * that was specified.  So we assume it's UTF-8 and hope for the best.
+	 */
+	if (*utf8 == 0) {
+	    strncpy(utf8, text, textlen);
+	}
+
+	return utf8;
+}
+
+static int gaim_parseaiminfo(aim_session_t *sess, aim_frame_t *fr, ...)
+{
+	struct gaim_connection *gc = sess->aux_data;
+	va_list ap;
+	aim_userinfo_t *userinfo;
+	guint16 infotype;
+	char *text_encoding = NULL, *text = NULL, *extracted_encoding = NULL;
+	guint16 text_length;
+	char *utf8 = NULL;
+
+	va_start(ap, fr);
+	userinfo = va_arg(ap, aim_userinfo_t *);
+	infotype = va_arg(ap, int);
+	text_encoding = va_arg(ap, char*);
+	text = va_arg(ap, char*);
+	text_length = va_arg(ap, int);
+	va_end(ap);
+
+	if(text_encoding)
+		extracted_encoding = oscar_encoding_extract(text_encoding);
+	if(infotype == AIM_GETINFO_GENERALINFO) {
+		/*Display idle time*/
+		char buff[256];
+		struct tm idletime;
+		if(userinfo->idletime) {
+			memset(&idletime, 0, sizeof(struct tm));
+			idletime.tm_mday = (userinfo->idletime / 60) / 24;
+			idletime.tm_hour = (userinfo->idletime / 60) % 24;
+			idletime.tm_min = userinfo->idletime % 60;
+			idletime.tm_sec = 0;
+			strftime(buff, 256, _("%d days %H hours %M minutes"), &idletime);
+			serv_got_crap(gc, "%s: %s", _("Idle Time"), buff);
+		}
+		
+		if(text) {
+			utf8 = oscar_encoding_to_utf8(extracted_encoding, text, text_length);
+			serv_got_crap(gc, "%s\n%s", _("User Info"), utf8);
+		} else {
+			serv_got_crap(gc, _("No user info available."));
+		}
+	} else if(infotype == AIM_GETINFO_AWAYMESSAGE && userinfo->flags & AIM_FLAG_AWAY) {
+		utf8 = oscar_encoding_to_utf8(extracted_encoding, text, text_length);
+		serv_got_crap(gc, "%s\n%s", _("Away Message"), utf8);
+	}
+
+	g_free(utf8);
+   
+	return 1;
+}
+
+int gaim_parsemtn(aim_session_t *sess, aim_frame_t *fr, ...)
+{
+	struct gaim_connection * gc = sess->aux_data;
+	va_list ap;
+	guint16 type1, type2;
+	char * sn;
+
+	va_start(ap, fr);
+	type1 = va_arg(ap, int);
+	sn = va_arg(ap, char*);
+	type2 = va_arg(ap, int);
+	va_end(ap);
+    
+	if(type2 == 0x0002) {
+		/* User is typing */
+		serv_got_typing(gc, sn, 0, 1);
+	} 
+	else if (type2 == 0x0001) {
+		/* User has typed something, but is not actively typing (stale) */
+		serv_got_typing(gc, sn, 0, 2);
+	}
+	else {
+		/* User has stopped typing */
+		serv_got_typing(gc, sn, 0, 0);
+	}        
+	
+	return 1;
+}
+
 static char *oscar_get_status_string( struct gaim_connection *gc, int number )
 {
 	struct oscar_data *od = gc->proto_data;
@@ -2314,6 +2494,144 @@ static char *oscar_get_status_string( struct gaim_connection *gc, int number )
 	}
 }
 
+int oscar_send_typing(struct gaim_connection *gc, char * who, int typing)
+{
+	struct oscar_data *od = gc->proto_data;
+	return( aim_im_sendmtn(od->sess, 1, who, typing ? 0x0002 : 0x0000) );
+}
+
+int oscar_chat_send(struct gaim_connection * gc, int id, char *message)
+{
+	struct oscar_data * od = (struct oscar_data*)gc->proto_data;
+	struct chat_connection * ccon;
+	int ret;
+	guint8 len = strlen(message);
+	char *s;
+	
+	if(!(ccon = find_oscar_chat(gc, id)))
+		return -1;
+	  	
+	for (s = message; *s; s++)
+		if (*s & 128)
+			break;
+	  	
+	/* Message contains high ASCII chars, time for some translation! */
+	if (*s) {
+		s = g_malloc(BUF_LONG);
+		/* Try if we can put it in an ISO8859-1 string first.
+		   If we can't, fall back to UTF16. */
+		if ((ret = do_iconv("UTF-8", "ISO8859-1", message, s, len, BUF_LONG)) >= 0) {
+			len = ret;
+		} else if ((ret = do_iconv("UTF-8", "UNICODEBIG", message, s, len, BUF_LONG)) >= 0) {
+			len = ret;
+		} else {
+			/* OOF, translation failed... Oh well.. */
+			g_free( s );
+			s = message;
+		}
+	} else {
+		s = message;
+	}
+	  	
+	ret = aim_chat_send_im(od->sess, ccon->conn, AIM_CHATFLAGS_NOREFLECT, s, len);
+	  	
+	if (s != message) {	
+		g_free(s);
+  }
+  
+  return (ret >= 0);
+}
+
+void oscar_chat_invite(struct gaim_connection * gc, int id, char *message, char *who)
+{
+	struct oscar_data * od = (struct oscar_data *)gc->proto_data;
+	struct chat_connection *ccon = find_oscar_chat(gc, id);
+	
+	if (ccon == NULL)
+		return;
+	
+	aim_chat_invite(od->sess, od->conn, who, message ? message : "",
+					ccon->exchange, ccon->name, 0x0);
+}
+
+void oscar_chat_kill(struct gaim_connection *gc, struct chat_connection *cc)
+{
+	struct oscar_data *od = (struct oscar_data *)gc->proto_data;
+
+	/* Notify the conversation window that we've left the chat */
+	serv_got_chat_left(gc, cc->id);
+
+	/* Destroy the chat_connection */
+	od->oscar_chats = g_slist_remove(od->oscar_chats, cc);
+	if (cc->inpa > 0)
+		gaim_input_remove(cc->inpa);
+	aim_conn_kill(od->sess, &cc->conn);
+	g_free(cc->name);
+	g_free(cc->show);
+	g_free(cc);
+}
+
+void oscar_chat_leave(struct gaim_connection * gc, int id)
+{
+	struct chat_connection * ccon = find_oscar_chat(gc, id);
+
+	if(ccon == NULL)
+		return;
+
+	oscar_chat_kill(gc, ccon);
+}
+
+int oscar_chat_join(struct gaim_connection * gc, char * name)
+{
+    struct oscar_data * od = (struct oscar_data *)gc->proto_data;
+	
+	aim_conn_t * cur;
+
+	if((cur = aim_getconn_type(od->sess, AIM_CONN_TYPE_CHATNAV))) {
+	
+		return (aim_chatnav_createroom(od->sess, cur, name, 4) == 0);
+	
+	} else {
+		struct create_room * cr = g_new0(struct create_room, 1);
+		cr->exchange = 4;
+		cr->name = g_strdup(name);
+		od->create_rooms = g_slist_append(od->create_rooms, cr);
+		aim_reqservice(od->sess, od->conn, AIM_CONN_TYPE_CHATNAV);
+		return 1;
+	}
+}
+
+int oscar_chat_open(struct gaim_connection * gc, char *who)
+{
+	struct oscar_data * od = (struct oscar_data *)gc->proto_data;
+	int ret;
+	static int chat_id = 0;
+	char * chatname = g_new0(char, strlen(gc->username)+4);
+	
+	g_snprintf(chatname, strlen(gc->username) + 4, "%s%d", gc->username, chat_id++);
+  
+	ret = oscar_chat_join(gc, chatname);
+
+	aim_chat_invite(od->sess, od->conn, who, "", 4, chatname, 0x0);
+
+	g_free(chatname);
+	
+	return ret;
+}
+
+void oscar_accept_chat(gpointer w, struct aim_chat_invitation * inv)
+{
+	oscar_chat_join(inv->gc, inv->name);
+	g_free(inv->name);
+	g_free(inv);
+}
+
+void oscar_reject_chat(gpointer w, struct aim_chat_invitation * inv)
+{
+	g_free(inv->name);
+	g_free(inv);
+}
+
 static struct prpl *my_protocol = NULL;
 
 void oscar_init(struct prpl *ret) {
@@ -2327,13 +2645,20 @@ void oscar_init(struct prpl *ret) {
 	ret->get_away = oscar_get_away;
 	ret->add_buddy = oscar_add_buddy;
 	ret->remove_buddy = oscar_remove_buddy;
+	ret->chat_send = oscar_chat_send;
+	ret->chat_invite = oscar_chat_invite;
+	ret->chat_leave = oscar_chat_leave;
+	ret->chat_open = oscar_chat_open;
 	ret->add_permit = oscar_add_permit;
 	ret->add_deny = oscar_add_deny;
 	ret->rem_permit = oscar_rem_permit;
 	ret->rem_deny = oscar_rem_deny;
 	ret->set_permit_deny = oscar_set_permit_deny;
 	ret->keepalive = oscar_keepalive;
+	ret->cmp_buddynames = aim_sncmp;
 	ret->get_status_string = oscar_get_status_string;
 
+	ret->send_typing = oscar_send_typing;
+
 	my_protocol = ret;
 }
