@@ -194,15 +194,15 @@ static int byahoo_send_typing( struct gaim_connection *gc, char *who, int typing
 static void byahoo_set_away( struct gaim_connection *gc, char *state, char *msg )
 {
 	struct byahoo_data *yd = (struct byahoo_data *) gc->proto_data;
-
+	
 	gc->away = NULL;
-
+	
 	if (msg)
 	{
 		yd->current_status = YAHOO_STATUS_CUSTOM;
 		gc->away = "";
 	}
-	else if (state)
+	if( state )
 	{
 		gc->away = "";
 		if( g_strcasecmp(state, "Available" ) == 0 )
@@ -244,8 +244,10 @@ static void byahoo_set_away( struct gaim_connection *gc, char *state, char *msg
 		yd->current_status = YAHOO_STATUS_IDLE;
 	else
 		yd->current_status = YAHOO_STATUS_AVAILABLE;
-	
-	yahoo_set_away( yd->y2_id, yd->current_status, msg, gc->away != NULL );
+	if(yd->current_status == YAHOO_STATUS_INVISIBLE)
+		yahoo_set_away( yd->y2_id, yd->current_status, NULL, gc->away != NULL );
+	else
+		yahoo_set_away( yd->y2_id, yd->current_status, msg, gc->away != NULL );
 }
 
 static GList *byahoo_away_states( struct gaim_connection *gc )
@@ -416,6 +418,7 @@ void byahoo_init( struct prpl *ret )
 	ret->chat_invite = byahoo_chat_invite;
 	ret->chat_leave = byahoo_chat_leave;
 	ret->chat_open = byahoo_chat_open;
+	ret->cmp_buddynames = g_strcasecmp;
 	
 	my_protocol = ret;
 }
@@ -639,8 +642,14 @@ void ext_yahoo_got_file( int id, char *who, char *url, long expires, char *msg,
 void ext_yahoo_typing_notify( int id, char *who, int stat )
 {
 	struct gaim_connection *gc = byahoo_get_gc_by_id( id );
-	
-	serv_got_typing( gc, who, 1 );
+	if (stat == 1) {
+		/* User is typing */
+		serv_got_typing( gc, who, 1, 1 );
+	}
+	else {
+		/* User stopped typing */
+		serv_got_typing( gc, who, 1, 0 );
+	}
 }
 
 void ext_yahoo_system_message( int id, char *msg )
@@ -849,8 +858,6 @@ void ext_yahoo_conf_userjoin( int id, char *who, char *room )
 	
 	if( c )
 		add_chat_buddy( c, who );
-	else if( set_getint( gc->irc, "debug" ) )
-		serv_got_crap( gc, "Got ext_yahoo_conf_userjoin() from %s for unknown conference %s", who, room );
 }
 
 void ext_yahoo_conf_userleave( int id, char *who, char *room )
@@ -862,8 +869,6 @@ void ext_yahoo_conf_userleave( int id, char *who, char *room )
 	
 	if( c )
 		remove_chat_buddy( c, who, "" );
-	else if( set_getint( gc->irc, "debug" ) )
-		serv_got_crap( gc, "Got ext_yahoo_conf_userleave() from %s for unknown conference %s", who, room );
 }
 
 void ext_yahoo_conf_message( int id, char *who, char *room, char *msg, int utf8 )
